@@ -107,7 +107,15 @@ func parsePropertyType(propValue map[string]interface{}) string {
 			if !ok {
 				log.Fatal("missing additionalProperties field when type=object:", propValue)
 			}
-			valueType := parsePropertyType(additionalProps.(map[string]interface{}))
+			var valueType string
+			switch actual := additionalProps.(type) {
+			case bool:
+				valueType = "interface{}"
+			case map[string]interface{}:
+				valueType = parsePropertyType(actual)
+			default:
+				log.Fatal("unexpected additionalProperties value:", additionalProps)
+			}
 			return fmt.Sprintf("map[string]%v", valueType)
 		case "interface{}":
 			return "interface{}"
@@ -165,7 +173,7 @@ func maybeParseInheritance(descMap map[string]json.RawMessage) (baseTypeName str
 // emitToplevelType emits a single type into a string. It takes the type name
 // and a serialized json object representing the type. The json representation
 // will have fields: "type", "properties" etc.
-func emitToplevelType(typeName string, descJson json.RawMessage) string {
+func emitToplevelType(typeName string, descJson json.RawMessage, goTypeIsStruct map[string]bool) string {
 	var b strings.Builder
 	var baseType string
 
@@ -281,7 +289,7 @@ func emitToplevelType(typeName string, descJson json.RawMessage) string {
 				bodyTypeName = parseRef(ref)
 			} else {
 				bodyTypeName = typeName + "Body"
-				bodyType = emitToplevelType(bodyTypeName, propsMapOfJson["body"])
+				bodyType = emitToplevelType(bodyTypeName, propsMapOfJson["body"], goTypeIsStruct)
 			}
 
 			if requiredMap["body"] {
@@ -313,15 +321,21 @@ func emitToplevelType(typeName string, descJson json.RawMessage) string {
 				jsonTag += "\"`"
 			} else if typeName == "ErrorMessage" && propName == "showUser" {
 				// For launch/attach errors, vscode will treat omitted values the same way as true,
-				// so to supress visible reporting, we must report false explicitly.
+				// so to suppress visible reporting, we must report false explicitly.
 				jsonTag += "\"`"
 			} else {
 				jsonTag += ",omitempty\"`"
-			}
-			if typeName == "Breakpoint" && goType == "Source" {
-				// Make the Source field in Breakpoint a pointer since a DAP client may distinguish
-				// between zero and nil source values.
-				goType = "*" + goType
+				// If the field should be omitted when empty and is a struct type in Go, make it a pointer,
+				// because non-pointer structs get initialized with default values in Go (and not nil), and
+				// are then indistinguishable from structs with values actually set to zero when serializing
+				// to JSON. Making them a pointer makes them initialize to nil, which is then indeed omitted
+				// during serialization.
+				if _, ok := propDesc["$ref"]; ok {
+					// If we have a ref, then goType is the parsed ref
+					if goTypeIsStruct[goType] {
+						goType = "*" + goType
+					}
+				}
 			}
 			fmt.Fprintf(&b, "\t%s %s %s\n", goFieldName(propName), goType, jsonTag)
 
@@ -511,21 +525,21 @@ type Message interface {
 	GetSeq() int
 }
 
-// RequestMessage is an interface implemented by all Request-types. 
+// RequestMessage is an interface implemented by all Request-types.
 type RequestMessage interface {
 	Message
 	// GetRequest provides access to the embedded Request.
 	GetRequest() *Request
 }
 
-// ResponseMessage is an interface implemented by all Response-types. 
+// ResponseMessage is an interface implemented by all Response-types.
 type ResponseMessage interface {
 	Message
 	// GetResponse provides access to the embedded Response.
 	GetResponse() *Response
 }
 
-// EventMessage is an interface implemented by all Event-types. 
+// EventMessage is an interface implemented by all Event-types.
 type EventMessage interface {
 	Message
 	// GetEvent provides access to the embedded Event.
@@ -584,6 +598,27 @@ func main() {
 		log.Fatal(err)
 	}
 
+	goTypesIsStruct := make(map[string]bool)
+	for typeName, descJson := range typeMap {
+		var descMap map[string]json.RawMessage
+		if err := json.Unmarshal(descJson, &descMap); err != nil {
+			log.Fatal(err)
+		}
+		_, descMap = maybeParseInheritance(descMap)
+
+		typeJson, ok := descMap["type"]
+		if !ok {
+			log.Fatal("want description to have 'type', got ", descMap)
+		}
+
+		var descTypeString string
+		if err := json.Unmarshal(typeJson, &descTypeString); err != nil {
+			log.Fatal(err)
+		}
+
+		goTypesIsStruct[replaceGoTypename(typeName)] = descTypeString == "object"
+	}
+
 	var b strings.Builder
 	b.WriteString(preamble)
 
@@ -595,7 +630,7 @@ func main() {
 	var requests, responses, events []string
 	for _, typeName := range typeNames {
 		if _, ok := typesExcludeList[typeName]; !ok {
-			b.WriteString(emitToplevelType(replaceGoTypename(typeName), typeMap[typeName]))
+			b.WriteString(emitToplevelType(replaceGoTypename(typeName), typeMap[typeName], goTypesIsStruct))
 			b.WriteString("\n")
 		}
 
