@@ -35,7 +35,7 @@ var errorResponseStruct = ErrorResponse{
 		Message:    "Unable to produce stack trace: \"{e}\"",
 	},
 	Body: ErrorResponseBody{
-		Error: ErrorMessage{
+		Error: &ErrorMessage{
 			Id:        2004,
 			Format:    "Unable to produce stack trace: \"{e}\"",
 			Variables: map[string]string{"e": "Unknown goroutine 1"},
@@ -49,7 +49,7 @@ var errorResponseStruct = ErrorResponse{
 var cancelRequestString = `{"seq":25,"type":"request","command":"cancel","arguments":{"requestId":24}}`
 var cancelRequestStruct = CancelRequest{
 	Request:   *newRequest(25, "cancel"),
-	Arguments: CancelArguments{RequestId: 24},
+	Arguments: &CancelArguments{RequestId: 24},
 }
 
 var cancelResponseString = `{"seq":25,"type":"response","request_seq":26,"command":"cancel","success":true}`
@@ -125,7 +125,7 @@ var initializeResponseStruct = InitializeResponse{
 var configurationDoneRequestString = `{"seq":2,"type":"request","command":"configurationDone"}`
 var configurationDoneRequestStruct = ConfigurationDoneRequest{
 	Request:   *newRequest(2, "configurationDone"),
-	Arguments: ConfigurationDoneArguments{},
+	Arguments: nil,
 }
 
 var configurationDoneResponseString = `{"seq":2,"type":"response","request_seq":3,"command":"configurationDone","success":true}`
@@ -155,10 +155,10 @@ var attachResponseStruct = AttachResponse{Response: *newResponse(4, 5, "attach",
 
 // -------- Restart
 
-var restartRequestString = `{"seq":5,"type":"request","command":"restart","arguments":{}}`
+var restartRequestString = `{"seq":5,"type":"request","command":"restart"}`
 var restartRequestStruct = RestartRequest{
 	Request:   *newRequest(5, "restart"),
-	Arguments: RestartArguments{},
+	Arguments: nil,
 }
 
 var restartResponseString = `{"seq":5,"type":"response","request_seq":6,"command":"restart","success":true}`
@@ -169,7 +169,7 @@ var restartResponseStruct = RestartResponse{Response: *newResponse(5, 6, "restar
 var disconnectRequestString = `{"seq":6,"type":"request","command":"disconnect","arguments":{"restart":true}}`
 var disconnectRequestStruct = DisconnectRequest{
 	Request:   *newRequest(6, "disconnect"),
-	Arguments: DisconnectArguments{Restart: true},
+	Arguments: &DisconnectArguments{Restart: true},
 }
 
 var disconnectResponseString = `{"seq":6,"type":"response","request_seq":7,"command":"disconnect","success":true}`
@@ -180,7 +180,7 @@ var disconnectResponseStruct = DisconnectResponse{Response: *newResponse(6, 7, "
 var terminateRequestString = `{"seq":7,"type":"request","command":"terminate","arguments":{"restart":true}}`
 var terminateRequestStruct = TerminateRequest{
 	Request:   *newRequest(7, "terminate"),
-	Arguments: TerminateArguments{Restart: true},
+	Arguments: &TerminateArguments{Restart: true},
 }
 
 var terminateResponseString = `{"seq":7,"type":"response","request_seq":8,"command":"terminate","success":true}`
@@ -191,7 +191,7 @@ var terminateResponseStruct = TerminateResponse{Response: *newResponse(7, 8, "te
 var breakpointLocationsRequestString = `{"seq":8,"type":"request","command":"breakpointLocations","arguments":{"source":{"name":"hello.go","path":"/Users/foo/go/src/hello/hello.go"},"line":10}}`
 var breakpointLocationsRequestStruct = BreakpointLocationsRequest{
 	Request: *newRequest(8, "breakpointLocations"),
-	Arguments: BreakpointLocationsArguments{
+	Arguments: &BreakpointLocationsArguments{
 		Source: Source{Name: "hello.go", Path: "/Users/foo/go/src/hello/hello.go"},
 		Line:   10,
 	},
@@ -411,7 +411,7 @@ var stackTraceResponseStruct = StackTraceResponse{
 		StackFrames: []StackFrame{
 			{
 				Id: 1000,
-				Source: Source{
+				Source: &Source{
 					Name:            "hello.go",
 					Path:            "/Users/foo/go/src/hello/hello.go",
 					SourceReference: 0,
@@ -422,7 +422,7 @@ var stackTraceResponseStruct = StackTraceResponse{
 			},
 			{
 				Id: 1001,
-				Source: Source{
+				Source: &Source{
 					Name:            "proc.go",
 					Path:            "/usr/local/go/src/runtime/proc.go",
 					SourceReference: 0,
@@ -433,7 +433,7 @@ var stackTraceResponseStruct = StackTraceResponse{
 			},
 			{
 				Id: 1002,
-				Source: Source{
+				Source: &Source{
 					Name:            "asm_amd64.s",
 					Path:            "/usr/local/go/src/runtime/asm_amd64.s",
 					SourceReference: 0,
@@ -573,10 +573,10 @@ var modulesResponseStruct = ModulesResponse{
 
 // -------- LoadedSources
 
-var loadedSourcesRequestString = `{"seq":31,"type":"request","command":"loadedSources","arguments":{}}`
+var loadedSourcesRequestString = `{"seq":31,"type":"request","command":"loadedSources"}`
 var loadedSourcesRequestStruct = LoadedSourcesRequest{
 	Request:   *newRequest(31, "loadedSources"),
-	Arguments: LoadedSourcesArguments{},
+	Arguments: nil,
 }
 
 var loadedSourcesResponseString = `{"seq":31,"type":"response","request_seq":32,"command":"loadedSources","success":true,"body":{"sources":[{"name":"hello.go","path":"/Users/foo/go/src/hello/hello.go"}]}}`
@@ -747,6 +747,22 @@ var disassembleResponseStruct = DisassembleResponse{
 	},
 }
 
+// -------- StartDebugging
+
+var startDebuggingRequestString = `{"seq":39,"type":"request","command":"startDebugging","arguments":{"request":"launch","configuration":{"any":true}}}`
+var startDebuggingRequestStruct = StartDebuggingRequest{
+	Request: *newRequest(39, "startDebugging"),
+	Arguments: StartDebuggingRequestArguments{
+		Request:       "launch",
+		Configuration: map[string]interface{}{"any": true},
+	},
+}
+
+var startDebuggingResponseString = `{"seq":39,"type":"response","request_seq":40,"command":"startDebugging","success":true}`
+var startDebuggingResponseStruct = StartDebuggingResponse{
+	Response: *newResponse(39, 40, "startDebugging", true),
+}
+
 // -------- Events --------
 
 var initializedEventString = `{"seq":1,"type":"event","event":"initialized"}`
@@ -820,7 +836,7 @@ var capabilitiesEventStruct = CapabilitiesEvent{
 	Body:  CapabilitiesEventBody{Capabilities: Capabilities{SupportsFunctionBreakpoints: true}},
 }
 
-func Test_DecodeProtocolMessage(t *testing.T) {
+func TestDecodeProtocolMessage(t *testing.T) {
 	// Sometimes partial messages can be returned on error, but
 	// the user should not rely on those and just check err itself.
 	// Hence the test will not check those.
@@ -882,6 +898,7 @@ func Test_DecodeProtocolMessage(t *testing.T) {
 		{exceptionInfoRequestString, &exceptionInfoRequestStruct, noError},
 		{readMemoryRequestString, &readMemoryRequestStruct, noError},
 		{disassembleRequestString, &disassembleRequestStruct, noError},
+		{startDebuggingRequestString, &startDebuggingRequestStruct, noError},
 		// Response
 		{`{"type":"response","success":true, "seq": 77}`, msgIgnoredOnError, "Response command '' is not supported (seq: 77)"},
 		{errorResponseString, &errorResponseStruct, noError},
@@ -926,6 +943,7 @@ func Test_DecodeProtocolMessage(t *testing.T) {
 		{exceptionInfoResponseString, &exceptionInfoResponseStruct, noError},
 		{readMemoryResponseString, &readMemoryResponseStruct, noError},
 		{disassembleResponseString, &disassembleResponseStruct, noError},
+		{startDebuggingResponseString, &startDebuggingResponseStruct, noError},
 		// Event
 		{`{"type":"event", "seq": 8}`, msgIgnoredOnError, "Event event '' is not supported (seq: 8)"},
 		{initializedEventString, &initializedEventStruct, noError},
@@ -941,6 +959,7 @@ func Test_DecodeProtocolMessage(t *testing.T) {
 		{processEventString, &processEventStruct, noError},
 		{capabilitiesEventString, &capabilitiesEventStruct, noError},
 	}
+
 	for _, test := range tests {
 		t.Run(test.data, func(t *testing.T) {
 			msg, err := DecodeProtocolMessage([]byte(test.data))
@@ -962,6 +981,75 @@ func Test_DecodeProtocolMessage(t *testing.T) {
 	}
 }
 
+// -------- Custom Request/Response and Event --------
+
+type customRequest struct {
+	Request
+	Body string `json:"body"`
+}
+
+func (r *customRequest) GetRequest() *Request { return &r.Request }
+
+type customResponse struct {
+	Response
+	Body string `json:"body"`
+}
+
+func (r *customResponse) GetResponse() *Response { return &r.Response }
+
+var customRequestString = `{"seq":40,"type":"request","command":"customReq","body":"242424"}`
+var customRequestStruct = customRequest{
+	Request: *newRequest(40, "customReq"),
+	Body:    "242424",
+}
+
+var customResponseString = `{"seq":40,"type":"response","request_seq":41,"command":"customReq","success":true,"body":"424242"}`
+var customResponseStruct = customResponse{
+	Response: *newResponse(40, 41, "customReq", true),
+	Body:     "424242",
+}
+
+type customEvent struct {
+	Event
+	Body int `json:"body"`
+}
+
+func (e *customEvent) GetEvent() *Event { return &e.Event }
+
+var customEventString = `{"seq":13,"type":"event","event":"customEvt","body":42}`
+var customEventStruct = customEvent{
+	Event: *newEvent(13, "customEvt"),
+	Body:  42,
+}
+
+func TestDecodeProtocolMessage_Custom(t *testing.T) {
+	tests := []struct {
+		data    string
+		wantMsg Message
+	}{
+		{customRequestString, &customRequestStruct},
+		{customResponseString, &customResponseStruct},
+		{customEventString, &customEventStruct},
+	}
+
+	codec := NewCodec()
+	codec.RegisterRequest("customReq", func() Message { return new(customRequest) }, func() Message { return new(customResponse) })
+	codec.RegisterEvent("customEvt", func() Message { return new(customEvent) })
+	for _, test := range tests {
+		t.Run(test.data, func(t *testing.T) {
+			msg, err := codec.DecodeMessage([]byte(test.data))
+			if err != nil { // Decoding error
+				t.Fatalf("codec.DecodeMessage() failed with %v", err)
+			}
+			got, _ := json.Marshal(msg)
+			want, _ := json.Marshal(test.wantMsg)
+			if !reflect.DeepEqual(msg, test.wantMsg) { // Check result
+				t.Errorf("\ngot message\n%s\nwant\n%s", got, want)
+			}
+		})
+	}
+}
+
 // newRequest builds a Request struct with the specified fields.
 func newRequest(seq int, command string) *Request {
 	return &Request{
